/*=====================================================================
File: CSVDocument.cs

Summary: 

Date : Septembre 2007

Origin: Open Cellar ! Project

Revision:

Aut: Matthieu DUCROCQ

---------------------------------------------------------------------
Copyright (C) Matthieu DUCROCQ
=====================================================================*/
using System;
using System.IO;
using System.Text;
using OpenCellar.Framework.Core;

namespace OpenCellar.Import.Framework
{
	/// <summary>
	/// CSVDocument
	/// </summary>
    public class CSVDocument : Document
    {
        /// <summary>
        /// Initialise une instance de classe CSVDocument.
        /// </summary>
        /// <param name="cellar"></param>
        /// <param name="filePath"></param>
        public CSVDocument(Cellar cellar, string filePath)
            : base(cellar)
        { 
            // Vrifie les parametres
            if((filePath == null)
                | (filePath == string.Empty))
            {
                throw new ArgumentNullException("filePath");
            }

            if (!File.Exists(filePath))
            {
                throw new FileNotFoundException(filePath);
            }

            this.filePath = filePath;
        }

        private string filePath = null;

        /// <summary>
        /// Obtient l'emplacement du fichier.
        /// </summary>
        public string FilePath
        {
            get
            {
                return (this.filePath);
            }
        }

        /// <summary>
        /// Sparateur par dfaut.
        /// </summary>
        const char SEPARATOR = ';';

        private char separator = CSVDocument.SEPARATOR;

        /// <summary>
        /// Obtient ou dfinit le caractere de separateur de ligne.
        /// </summary>
        public char Separator
        {
            get
            {
                return (this.separator);
            }
            set
            {
                this.separator = value;
            }
        }

        private string[] columnNames;

        /// <summary>
        /// Obtient la liste des entetes.
        /// </summary>
        /// <returns></returns>
        public override string[] GetColumnNames()
        {
            // Vrifie si le document est ouvert
            if (!base.IsOpen)
            {
                throw new Exception("Open::false");
            }

            if (base.ParseError)
            {
                throw new Exception("ParseError::true");
            }

            return (columnNames);
        }

        /// <summary>
        /// Ouvre le document spcifi.
        /// </summary>
        /// <returns></returns>
        public override bool Open()
        {
            // Vrifie que le document ne soit pas deja ouvert.
            if (base.IsOpen)
            {
                return(!base.ParseError);
            }

            if (base.ParseError)
            {
                throw new Exception("ParseError::true");
            }

            // Ouverture du document
            bool success = __parse();
            
            // Affectation des tas
            base.IsOpen = true;
            base.ParseError = !success;
            return (success);
        }

        /// <summary>
        /// Analyse le document.
        /// </summary>
        /// <returns></returns>
        private bool __parse()
        {
            bool success = false;

            StreamReader reader = null;

            try
            {
                reader = new StreamReader(this.FilePath, Encoding.Default);
                string line = null;

                line = reader.ReadLine();
                if (line != null)
                {
                    // Entete de colonne.
                    CSVRow row = new CSVRow(this, 0, line);
                    columnNames = row.ToStringArray();
                }

                int lineNumber = 0;
                while ((line = reader.ReadLine()) != null)
                {
                    IRow row = new CSVRow(this, lineNumber, line);
                    this.Rows.Add(row);
					lineNumber++;
                }

                success = true;
            }
            catch (Exception ex)
            {
                System.Diagnostics.Trace.WriteLine("__parseError() :: ");
                System.Diagnostics.Trace.WriteLine(ex.ToString());
            }
            finally
            {
                if (reader != null)
                {
                    reader.Close();
                }
            }

            return (success);
        }
    }
}
