/*=====================================================================
File: Document.cs

Summary: 

Date : Septembre 2007

Origin: Open Cellar ! Project

Revision:

Aut: Matthieu DUCROCQ

---------------------------------------------------------------------
Copyright (C) Matthieu DUCROCQ
=====================================================================*/
using System;
using System.Collections;
using System.Reflection;
using OpenCellar.Framework.Core;

namespace OpenCellar.Import.Framework
{
    public class Document : IDocument
    {
        /// <summary>
        /// Constructeur statique de la classe
        /// Charge de charger toutes les colonnes disponibles.
        /// </summary>
        static Document()
        {
            System.Reflection.Assembly asm = System.Reflection.Assembly.GetExecutingAssembly();
            IEnumerator enumerator = asm.GetTypes().GetEnumerator();
            while (enumerator.MoveNext())
            {
                Type type = (Type)enumerator.Current;
                if ((type.GetInterface("IColumn") != null)
                    && (!type.IsAbstract))
                {
                    IColumn column = null;
                    try
                    {
                        column = (IColumn)Activator.CreateInstance(type);
                        __AvailableColumns.Add(column);
                    }
                    catch (Exception colEx)
                    {
                        System.Diagnostics.Trace.WriteLine("static_ctor::Document");
                        System.Diagnostics.Trace.WriteLine(colEx.ToString());
                    }
                    finally
                    {
                        column = null;
                    }
                }
            }
            asm = null;

            // Tri les colonnes.
            __AvailableColumns.Sort();
        }

        static ColumnCollection __AvailableColumns = new ColumnCollection(false);


        /// <summary>
        /// Initialise une instance de classe BaseDocument.
        /// </summary>
        protected Document(Cellar cellar)
        {
            if (cellar == null)
            {
                throw new ArgumentNullException("cellar");
            }

            this.cellar = cellar;

            __loadAvaibleColumns();
        }

        /// <summary>
        /// Charge la liste des colonnes disponibles.
        /// </summary>
        private void __loadAvaibleColumns()
        {
            for (int i = 0; i < Document.__AvailableColumns.Count; i++)
            {
                ((ColumnCollection)this.availableColumns).__Add(Document.__AvailableColumns[i].Clone());
            }
        }

        /// <summary>
        /// Ouvre le document.
        /// </summary>
        public virtual bool Open()
        {
            return (false);
        }

        private Cellar cellar = null;

        /// <summary>
        /// Obtient une rfrence vers la cave de destination.
        /// </summary>
        public Cellar Cellar
        {
            get 
            {
                return (this.cellar);
            }
        }

        /// <summary>
        /// Obtient la liste des colonnes.
        /// </summary>
        /// <returns>Tableau de chaine contenant les colonnes.</returns>
        public virtual string[] GetColumnNames()
        {
            throw new NotImplementedException();
        }

        private IRowCollection rows = new RowCollection();

        /// <summary>
        /// Obtient la liste des lignes du document.
        /// </summary>
        public IRowCollection Rows
        {
            get
            {
                return (this.rows);
            }
        }

        private IColumnCollection columns = new ColumnCollection(false);

        /// <summary>
        /// Obtient la liste des colonnes du document.
        /// </summary>
        public IColumnCollection Columns
        {
            get
            {
                return (this.columns);
            }
        }

        private IColumnCollection availableColumns = new ColumnCollection(true);

        /// <summary>
        /// Obtient la liste des colonnes disponibles.
        /// </summary>
        public IColumnCollection AvailableColumns
        {
            get
            {
                return (this.availableColumns);
            }
        }

        /// <summary>
        /// Procde  l'importation des donnes.
        /// </summary>
        public virtual void Process()
        {
            OnBeginProcess();

            // Tri les lments selon leur priorit
            ((ColumnCollection)this.columns).Sort();

            for (int iRows = 0; iRows < this.Rows.Count; iRows++)
            {
                IRow row = this.Rows[iRows];

                // Cration du vin
                Wine wine = (Wine)Cellar.CreateItem(ObjectType.Wine);

                for (int iColumns = 0; iColumns < this.Columns.Count; iColumns++)
                {
                    bool ret = this.Columns[iColumns].Process(wine, row);
                }

				Wine newWine = GetWine(wine);

				if(newWine == null)
				{

					// Enregistrement
					wine.Save();

					// Enregistre les files d'attente
					wine.PurchaseWorkqueue.Apply();
					wine.AssemblyWorkqueue.Apply();
					wine.RackItemWorkqueue.Apply();
				}
				else
				{
					// Fusion des proprits.
					Merge(wine, newWine);

					// Enregistrement
					newWine.Save();

					// Enregistre les files d'attente
					newWine.PurchaseWorkqueue.Apply();
					newWine.AssemblyWorkqueue.Apply();
					newWine.RackItemWorkqueue.Apply();
				}

                // Evenement
                OnProgressChanged(row);
            }

            OnEndProcess();
        }

		/// <summary>
		/// Fusionne les proprits des 2 vins.
		/// </summary>
		/// <param name="wine"></param>
		/// <param name="newWine"></param>
		protected virtual void Merge(Wine wine, Wine newWine)
		{
			if(wine.PurchaseWorkqueue.Count > 0)
			{
				newWine.PurchaseWorkqueue.Set(WorkqueueItemType.Add,
					wine.PurchaseWorkqueue[0].Item);
			}

			if(wine.RackItemWorkqueue.Count > 0)
			{
				newWine.RackItemWorkqueue.Set(WorkqueueItemType.Add,
					wine.RackItemWorkqueue[0].Item);
			}
		}

		/// <summary>
		/// Renvoit
		/// </summary>
		/// <param name="wine"></param>
		/// <returns></returns>
		protected virtual Wine GetWine(Wine wine)
		{
			ObjectCollection wines = this.Cellar.GetCollection((ushort)ObjectType.Wine);
			for (int i = 0 ; i < wines.Count; i++)
			{
				Wine oc_wine = (Wine)wines[i];
				if((oc_wine.Year == wine.Year)
					&& (oc_wine.WineColor == wine.WineColor)
					&& (oc_wine.Reference.Trim() == wine.Reference.Trim())
					&& (oc_wine.Appellation.Name.Trim().ToLower() == wine.Appellation.Name.Trim().ToLower())
					&& (oc_wine.Name.Trim().ToLower() == wine.Name.Trim().ToLower()))
				{
					return(oc_wine);
				}
			}
			return(null);
		}

        /// <summary>
        /// Se produit lorsque l'tat d'avancement est modifi.
        /// </summary>
        public event EventHandler ProgressChanged;

        /// <summary>
        /// Dclenche l'evenement Message.
        /// </summary>
        internal protected void OnProgressChanged(IRow sender)
        {
            if (ProgressChanged != null)
            {
                ProgressChanged(sender, EventArgs.Empty);
            }
        }

        /// <summary>
        /// Se produit lors du dmarrage de l'importation.
        /// </summary>
        public event EventHandler BeginProcess;

        /// <summary>
        /// Dclenche l'evenement BeginProcess.
        /// </summary>
        protected virtual void OnBeginProcess()
        {
            if (BeginProcess != null)
            {
                BeginProcess(this, EventArgs.Empty);
            }
        }

        /// <summary>
        /// Se produit lorsque le processus d'importation est termine.
        /// </summary>
        public event EventHandler EndProcess;

        /// <summary>
        /// Dclenche l'evenement EndProcess.
        /// </summary>
        protected virtual void OnEndProcess()
        {
            if (EndProcess != null)
            {
                EndProcess(this, EventArgs.Empty);
            }
        }

        private bool isOpen = false;

        /// <summary>
        /// Obtient ou dfinit si le document est ouvert.
        /// </summary>
        protected bool IsOpen
        {
            get
            {
                return (this.isOpen);
            }
            set
            {
                this.isOpen = value;
            }
        }

        private bool parseError = false;

        /// <summary>
        /// Obtient ou dfinit si le document a recontr des erreurs
        /// durant son analyse.
        /// </summary>
        protected bool ParseError
        {
            get
            {
                return (this.parseError);
            }
            set
            {
                this.parseError = value;
            }
        }
    }
}
